(****************************************************************************)   
(** <MKSoftware> - Software von Michael Khn fr den Unterricht *)   
(** www.mksoftware.de.vu *)
(** mail: mksoftware@bm-kuehn.de *)
(****************************************************************************)   
(* zufall.mod (Mrz 2002) *)
(* (c) by Michael Khn *)
(**                                                                          *)
(** Dieses Service-Modul darf fr Unterrichtszwecke frei verwendet werden.   *)
(**                                                                          *)
(** Fr Verbesserungsvorschlge bin ich dankbar.                             *)
(****************************************************************************)   

(** *)
(** <zufall.mod>*)
(** Zufallsgenerator fr Pow!*)
(** Der Algorithmus beruht auf dem Buch "Algorithmen und Datenstrukturen" 
    von N. Wirth.*)
(** Ein Randomize ist nicht ntig.*)

MODULE Zufall; (* zufall.mod: Zufallsgenerator *)
IMPORT Utils;

VAR z: LONGINT;
  
  
PROCEDURE RandomInt*(zahl:LONGINT): LONGINT;
(** Es wird eine Longint-'Zufallszahl' zwischen 0 und <zahl>-1 erzeugt.*)
(** *)
(** Aufruf z.B. x:= Zufall.RandomInt(6)+1; *)
(**   -> Hier wird eine Zufallszahl zwischen 1 und 6 ermittelt. *)
(** *)
(** Im Normalfall braucht man nur INTEGER-Werte statt LONGINT-Zahlen. 
    Dann muss der Aufruf noch mit SHORT(...) umklammert werden, z.B. *)
(** x:= SHORT(Zufall.RandomInt(6)+1); *)
  CONST a=16807;
        m=2147483647;
        q=m DIV a;
        r=m MOD a;
  VAR g: LONGINT;
  BEGIN
    g:=a*(z MOD q)-r*(z DIV q);
    IF g>0 
    THEN z:=g
    ELSE z:=g+m
    END;
    (* z*(1.0/m) zw. 0 und 1 *)
    RETURN  ENTIER(z*(1.0/m)*zahl)
END RandomInt;

PROCEDURE RandomReal*(): REAL;
(** Es wird eine Real-'Zufallszahl' zwischen 0 und 1 erzeugt.*)
(** *)
(** Aufruf z.B. a:= Zufall.RandomReal(); *)
  CONST a=16807;
        m=2147483647;
        q=m DIV a;
        r=m MOD a;
  VAR g: LONGINT;
  BEGIN
    g:=a*(z MOD q)-r*(z DIV q);
    IF g>0 
    THEN z:=g
    ELSE z:=g+m
    END;
    (* z*(1.0/m) zw. 0 und 1 *)
    RETURN  z/m
  END RandomReal;
  
PROCEDURE Init*;
(** Initialisiert den Zufallsgenerator. Dies mssen Sie im Normalfall nicht tun. 
    Nur wenn Sie bei mehreren Programmlufen jedes Mal dieselben "Zufallszahlen" 
    erhalten mchten, mssen Sie einmal bei Programmstart <Init> aufrufen. *)
BEGIN
  z:=314159; (* Zufallsgenerator initialisieren *)
END Init;

PROCEDURE Randomize;
(* Diese Prozedur gehrt zur Initialisierung des Zufallsgenerators. 
   Sie ist nicht ffentlich *)
VAR i: LONGINT;
    min,std,sek: INTEGER;
    zufi : REAL;
BEGIN
  Utils.GetTime(sek,min,std);
  FOR i:=1 TO sek*(min+sek) DO
    zufi:=RandomReal();
  END;
END Randomize;

BEGIN
  Init;
  Randomize;
END Zufall.
